<?php
/**
 * evilsentinel.class.php evilsentinel class file .
 * 
 * Contains the main evilsentinel system class .
 *
 * @author evilsocket <evilsocket@gmail.com>
 * @copyright Copyleft (c) 2007, evilsocket
 */

include_once "attacker.class.php";
include_once "report.class.php";
include_once "ait.class.php";
include_once "escore.php";

/**
 * The main evilsentinel system class .
 *
 * @package EvilSentinel
 */
class EvilSentinel
{
	/**
	 * Alarm state flag .
	 *
	 * @access private
	 * @var bool
	 */
	var $alarm_state   = false;
	/**
	 * The report object of the system .
	 *
	 * @access private
	 * @var Report
	 * @see report.class.php
	 */	
	var $report;

	/**
	 * Class constructor, all the security checking is made creating the object by this .
	 *
	 * @see main.php
	 */	
	function EvilSentinel()
	{
		global $es_settings;

		/* create global report object */
		$this->report = new Report();
		
		es_dispatch_event( "on_filtering_start" );

		if( $es_settings['es_filter_get'] ){
			/* loop each couple in GET */
			foreach( $_GET as $key => $value ){
				$value = trim(urldecode($value));
				/* loop each rule */
				foreach( $es_settings['es_rules'] as $rule ){	
					/* search a match with rule criteria */
					if( @eregi( $rule['criteria'], $value ) ){
						/* add description of matched criteria to report */
						$this->report->add( "description", "Found a probable {$rule['description']} => $key : $value" );	
						/* act accordingly to rule action */
						switch(strtolower($rule['action'])){
							case "block"    : $this->alarm_state = true; break;
							case "ignore"   : /* do nothing :P */        break;
							case "sanitize" : 
								/* sanitize accordingly to rule type */
								switch(strtolower($rule['type'])){
									case "xss" : $_GET[$key] = htmlentities($_GET[$key]);             break;
									case "rfi" : $_GET[$key] = str_replace("://","",$_GET[$key]);     break;
								    case "lfi" : $_GET[$key] = str_replace(".","",$_GET[$key]);       break;
									case "sql" : $_GET[$key] = mysql_real_escape_string($_GET[$key]); break;
								}
							break;
						}
					}
				}
			}
		}

		if( $es_settings['es_filter_post'] ){
			/* loop each couple in POST */
			foreach( $_POST as $key => $value ){
				$value = trim(@urldecode($value));
				/* loop each rule */
				foreach( $es_settings['es_rules'] as $rule ){
					/* search a match with rule criteria */
					if( @eregi( $rule['criteria'], $value ) ){
						/* add description of matched criteria to report */
						$this->report->add( "description", "Found a probable {$rule['description']} => $key : $value" );	
						/* act accordingly to rule action */
						switch(strtolower($rule['action'])){
							case "block"    : $this->alarm_state = true; break;
							case "ignore"   : /* do nothing :P */        break;
							case "sanitize" : 
								/* sanitize accordingly to rule type */
								switch(strtolower($rule['type'])){
									case "xss" : $_POST[$key] = htmlentities($_POST[$key]);             break;
									case "rfi" : $_POST[$key] = str_replace("://","",$_POST[$key]);     break;
								    case "lfi" : $_POST[$key] = str_replace(".","",$_POST[$key]);       break;
									case "sql" : $_POST[$key] = mysql_real_escape_string($_POST[$key]); break;
								}
							break;
						}
					}
				}
			}
		}

		if( $es_settings['es_filter_cookie'] ){
			/* loop each couple in COOKIE */
			foreach( $_COOKIE as $key => $value ){
				$value = trim(urldecode($value));
				/* loop each rule */
				foreach( $es_settings['es_rules'] as $rule ){
					/* search a match with rule criteria */
					if( @eregi( $rule['criteria'], $value ) ){
						/* add description of matched criteria to report */
						$this->report->add( "description", "Found a probable {$rule['description']} => $key : $value" );	
						/* act accordingly to rule action */
						switch(strtolower($rule['action'])){
							case "block"    : $this->alarm_state = true; break;
							case "ignore"   : /* do nothing :P */        break;
							case "sanitize" : 
								/* sanitize accordingly to rule type */
								switch(strtolower($rule['type'])){
									case "xss" : $_COOKIE[$key] = htmlentities($_COOKIE[$key]);             break;
									case "rfi" : $_COOKIE[$key] = str_replace("://","",$_COOKIE[$key]);     break;
								    case "lfi" : $_COOKIE[$key] = str_replace(".","",$_COOKIE[$key]);       break;
									case "sql" : $_COOKIE[$key] = mysql_real_escape_string($_COOKIE[$key]); break;
								}
							break;
						}
					}
				}
			}
		}

		es_dispatch_event( "on_filtering_end" );
		
		/* start referer check for administration pages */
		if( $es_settings['es_filter_referer'] ){
			if( @eregi( $es_settings['es_referer_trigger'],  $_SERVER['REQUEST_URI'] ) ){
				if( !strpos( $_SERVER['HTTP_REFERER'], $_SERVER['HTTP_HOST'] ) ){
					$this->report->add( "description", "Bad referer for '{$es_settings['es_referer_trigger']}' : ".$_SERVER['HTTP_REFERER'] );
					$this->alarm_state = true;
				}
			}
		}
		
		/* start/continue ip tracking if alarm state == true */
		if( $es_settings['es_ip_tracking'] ){
			$tracker = new AIT( $this->alarm_state, $_SERVER['REMOTE_ADDR'] );
		}
		
		if( $this->alarm_state ){
			/* compile report */
			$attacker = new Attacker();
			
			$this->report->add( "attacker_address", $attacker->ip_address );
			$this->report->add( "attacker_host",    $attacker->host_name  );
			$this->report->add( "attacker_referer", $attacker->referer    );
			$this->report->add( "attacker_agent",   $attacker->agent      );
			
			es_dispatch_event( "on_alarm_state", $this->report  );
			
			/* lg the attack into the database */
			$datetime    = mysql_real_escape_string($this->report->datetime());
			$address     = mysql_real_escape_string($attacker->ip_address);
			$host        = mysql_real_escape_string($attacker->host_name);
			$referer     = mysql_real_escape_string($attacker->referer);
			$agent       = mysql_real_escape_string($attacker->agent);
			$description = mysql_real_escape_string($this->report->get("description"));
			
			es_log_attack($datetime,$address,$host,$referer,$agent,$description);

			/* execute email notification */		
			if( $es_settings['es_email_notify'] ){
				es_dispatch_event( "on_notify_report", $es_settings['es_notify_email'], $this->report );
				es_send_report( $es_settings['es_notify_email'], "!!! EvilSentinel Alarm Report !!!", $this->report->toHtml() );
			}

			/* lock the system */
			es_dispatch_event( "on_lock_system" );
			es_set_session_report( $this->report->buffer() );
			/* destroy previous output buffered objects */
			@ob_end_clean();
			/* create sys lock output buffer */
			$sessionreport = $this->report->buffer();
			$htmlreport   = "";
			foreach( $sessionreport as $line ){
				$htmlreport .= htmlentities($line)."<br/>";
			}	
			$css = es_relative_path( "evilsentinel/style.css" );
			$img = es_relative_path( "evilsentinel/images/lock.png" );
			$htmloutput =<<<EOM
<html>
<head>
<link rel='StyleSheet' type='text/css'>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>!!! EvilSentinel Alarm !!!</title>
<style type="text/css">
</style>
<link href="$css" rel="stylesheet" type="text/css" />
</head>
<body class="thrColEls">
<br/><br/>
<table width="80%" border="0" align="center">
<tr>
	<td valign="top"><div class="blocktitle"><font color="#FF0000">!!! EVILSENTINEL ALARM !!!</font></div>
	<p>
		<br/>
		Alcuni dati che hai inviato a questo sito sono stati classificati come pericolosi dal sistema di sicurezza EvilSentinel, di conseguenza l'amministratore di questo sito è stato appena avvisato dell'accaduto tramite email con il report di tutti i dati a riguardo, compreso il tuo indirizzo ip .<br />
		Di seguito trovi le informazioni così come sono state inviate all'amministratore :
	</p> 
	<p>
		<font color="#FF0000">$htmlreport</font>
		<div align='center'><img src='$img'></div>
	</p>
	<p>
		Se reputi che ci sia stato un errore nell'interpretazione dei dati, puoi tranquillamente mandare un email all'amministratore del sito, il quale sarà lieto di chiarire con te l'accaduto .
	</p>
	<div class="blocktitle"></div>
</td>
</tr>
<tr>
	<td height="38" align='right'><em><a href="http://www.evilsocket.net">EvilSentinel Copyleft (c) 2007 by evilsocket@gmail.com</a></em></td>
</tr>
</table>
</body>
</html>
EOM;
			die($htmloutput);
		}
	}	
}

?>