<?php
/**
 * report.class.php reporting system class file .
 * 
 * Contains the reporting system class .
 *
 * @author evilsocket <evilsocket@gmail.com>
 * @copyright Copyleft (c) 2007, evilsocket
 */

include_once "escore.php";

/**
 * The reporting system class .
 *
 * @package Report
 */
class Report
{
	/**
	 * Date and time of the report .
	 *
	 * @access private
	 * @var string
	 */	
	var $datetime;
	/**
	 * Univoque timestamp of the report, used as part of the file name .
	 *
	 * @access private
	 * @var int
	 */	
	var $timestamp;
	/**
	 * Array of tags to use in the xml report file .
	 *
	 * @access private
	 * @var mixed
	 */		
	var $tags_buffer = array();
	/**
	 * Array of lines to use in the report .
	 *
	 * @access private
	 * @var mixed
	 */	
	var $line_buffer = array();
	
	/**
	 * Class constructor .
	 *
	 * @see EvilSentinel::EvilSentinel()
	 */	
	function Report(){
		$this->datetime  = date("D F d Y (H:m:s)",time());
		$this->timestamp = time().microtime();
	}
	
	/**
	 * Getter for $datetime .
	 *
	 * @access public
	 * @return string $datetime inner variable .
	 */	
	function datetime(){
		return $this->datetime;
	}

	/**
	 * Adds a tagged entry to the report buffers .
	 *
	 * @access public
	 * @param string $tag tag to use for the entry .
	 * @param string $line line of the entry .
	 */	
	function add( $tag, $line ){
		$this->tags_buffer[] = $tag;
		$this->line_buffer[] = ($line == "" ? "none" : $line);
	}
	
	/**
	 * Get value by tag .
	 *
	 * @access public
	 * @param string $tag tag to use for the entry .
	 * @return string null or the line found .
	 */
	function get( $tag ){
		$i = 0;
		foreach( $this->tags_buffer as $tagbuf ){
			if( $tagbuf == $tag ){
				return $this->line_buffer[$i];
			}
			$i++;
		}
	}
	
	/**
	 * Set value by tag .
	 *
	 * @access public
	 * @param string $tag tag to use for the entry .
	 */
	function set( $tag, $val ){
		$i = 0;
		foreach( $this->tags_buffer as $tagbuf ){
			if( $tagbuf == $tag ){
				$this->line_buffer[$i] = $val;
				return;
			}
			$i++;
		}
	}
	
	/**
	 * Append a line to the line buffer with tag .
	 *
	 * @access public
	 * @param string $tag tag to use for the entry .
	 * @param string $line line to append to the entry .
	 */
	function append( $tag, $line ){
		$i = 0;
		foreach( $this->tags_buffer as $tagbuf ){
			if( $tagbuf == $tag ){
				$this->line_buffer[$i] += ($line == "" ? "none" : $line);
			}
			$i++;
		}
	}
	
	/**
	 * Returns html body of the report
	 *
	 * @access public
	 * @return string the report html .
	 */	
	function toHtml(){
		$html = @file_get_contents( es_relative_path("evilsentinel/report_html_mail.tpl") );

		$html = str_replace( "#URL#",  "http://".$_SERVER['HTTP_HOST'], $html );
		$html = str_replace( "#DATE#", date("F/d/Y",time()), $html );
		$html = str_replace( "#TIME#", date("H:m:s",time()), $html );
		
		$i = 0;
		foreach( $this->tags_buffer as $tag ){
			switch( $tag ){
				case "description"      : $html = str_replace( "#DESCRIPTION#", htmlentities($this->line_buffer[$i]), $html ); break;
				case "attacker_address" : $html = str_replace( "#ADDRESS#",     htmlentities($this->line_buffer[$i]), $html ); break;
				case "attacker_host"    : $html = str_replace( "#HOST#",        htmlentities($this->line_buffer[$i]), $html ); break;
				case "attacker_referer" : $html = str_replace( "#REFERER#",     htmlentities($this->line_buffer[$i]), $html ); break;
				case "attacker_agent"   : $html = str_replace( "#AGENT#",       htmlentities($this->line_buffer[$i]), $html ); break;
			}
			
			$i++;
		}

		return $html;
	}
	
	/**
	 * Returns line buffer array .
	 *
	 * @access public
	 * @return mixed the line buffer array .
	 * @see EvilSentinel::report()
	 */	
	function buffer(){
		return $this->line_buffer;
	}
}
 
?>